package com.tzld.piaoquan.content.understanding.service.impl;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.content.understanding.common.enums.ExceptionEnum;
import com.tzld.piaoquan.content.understanding.common.exception.CommonException;
import com.tzld.piaoquan.content.understanding.model.dto.ContentUnderstandDTO;
import com.tzld.piaoquan.content.understanding.model.dto.TaskInfoDTO;
import com.tzld.piaoquan.content.understanding.model.dto.TaskResultDTO;
import com.tzld.piaoquan.content.understanding.model.param.ContentAnalyseParam;
import com.tzld.piaoquan.content.understanding.model.param.ContentUnderstandParam;
import com.tzld.piaoquan.content.understanding.service.ContentService;
import com.tzld.piaoquan.content.understanding.service.LoghubService;
import com.tzld.piaoquan.content.understanding.service.PipelineService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author supeng
 */
@Slf4j
@Service
public class ContentServiceImpl implements ContentService {

    @Autowired
    private LoghubService loghubService;

    @Autowired
    private PipelineService pipelineService;

    @Value("${video.analyse.pipelineId:1}")
    private Long videoAnalysePipelineId;

    @ApolloJsonValue("${content.understand.pipeline.config:{}}")
    private Map<String, Long> configMap;

    @Override
    public void analyse(ContentAnalyseParam param) {
        if (Objects.isNull(param) || Objects.isNull(param.getType())) {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }
        param.setPipelineId(videoAnalysePipelineId);
        pipelineService.execute(param);
    }

    @Override
    public void understand(ContentUnderstandParam param) {
        if (Objects.isNull(param) || Objects.isNull(param.getContentTypes()) || param.getContentTypes().isEmpty()
                || Objects.isNull(param.getContentInfo())) {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }
        //生成jobId
        String jobId = UUID.randomUUID().toString().replace("-", "") + System.currentTimeMillis();
        for (Integer contentType : param.getContentTypes()) {
            Long pipelineId = configMap.get(contentType.toString());
            if (Objects.isNull(pipelineId)) {
                //TODO 日志上报 && 记录状态
                log.info("jobId = {}", jobId);
                continue;
            }
            ContentUnderstandDTO dto = new ContentUnderstandDTO();
            BeanUtils.copyProperties(param.getContentInfo(), dto);
            dto.setPipelineId(pipelineId);
            dto.setContentType(contentType);
            pipelineService.execute(dto);
        }
    }

    @Override
    public TaskResultDTO submitTasks(ContentUnderstandParam param) {
        TaskResultDTO taskResultDTO = new TaskResultDTO();
        List<TaskInfoDTO> taskInfoDTOS = new ArrayList<>();
        for (Integer contentType : param.getContentTypes()) {
            Long pipelineId = configMap.get(contentType.toString());
            if (Objects.isNull(pipelineId)) {
                //TODO 日志上报 && 记录状态
                continue;
            }
            //生成 taskId
            String taskId = UUID.randomUUID().toString().replace("-", "") + System.currentTimeMillis();
            log.info("contentType = {} pipelineId = {} taskId = {}", contentType, pipelineId, taskId);
            ContentUnderstandDTO dto = new ContentUnderstandDTO();
            BeanUtils.copyProperties(param.getContentInfo(), dto);
            dto.setPipelineId(pipelineId);
            dto.setContentType(contentType);
            pipelineService.execute(dto);
        }
        taskResultDTO.setTaskList(taskInfoDTOS);
        return taskResultDTO;
    }

    @Override
    public TaskResultDTO queryTasks(ContentUnderstandParam param) {
        return null;
    }
}
